"""Test the deferred import error mechanism"""


# See https://github.com/spyder-ide/qtpy/pull/387/


import pytest

from qtpy import QtModuleNotInstalledError


def test_missing_optional_deps():
    """Test importing a module that uses the deferred import error mechanism"""
    from . import optional_deps

    assert optional_deps.ExampleClass is not None

    with pytest.raises(QtModuleNotInstalledError) as excinfo:
        from .optional_deps import MissingClass

    msg = "The optional_dep.MissingClass module was not found. It must be installed separately as test_package_please_ignore."
    assert msg == str(excinfo.value)



"""Utility functions for tests."""

import os


def using_conda():
    return os.environ.get("USE_CONDA", "Yes") == "Yes"


def not_using_conda():
    return os.environ.get("USE_CONDA", "No") == "No"



"""Test QtWinExtras."""

import sys

import pytest

from qtpy import PYQT6, PYSIDE2, PYSIDE6
from qtpy.tests.utils import using_conda


@pytest.mark.skipif(
    PYQT6 or PYSIDE6,
    reason="Not available on Qt6-based bindings",
)
@pytest.mark.skipif(
    sys.platform != "win32" or using_conda(),
    reason="Only available in Qt5 bindings > 5.9 with pip on Windows in CIs",
)
def test_qtwinextras():
    """Test the qtpy.QtWinExtras namespace"""
    from qtpy import QtWinExtras

    assert QtWinExtras.QWinJumpList is not None
    assert QtWinExtras.QWinJumpListCategory is not None
    assert QtWinExtras.QWinJumpListItem is not None
    assert QtWinExtras.QWinTaskbarButton is not None
    assert QtWinExtras.QWinTaskbarProgress is not None
    assert QtWinExtras.QWinThumbnailToolBar is not None
    assert QtWinExtras.QWinThumbnailToolButton is not None
    if not PYSIDE2:  # See https://bugreports.qt.io/browse/PYSIDE-1047
        assert QtWinExtras.QtWin is not None

    if PYSIDE2:
        assert QtWinExtras.QWinColorizationChangeEvent is not None
        assert QtWinExtras.QWinCompositionChangeEvent is not None
        assert QtWinExtras.QWinEvent is not None



"""Test QtWidgets."""
import contextlib
import sys
from time import sleep

import pytest
from pytestqt.exceptions import TimeoutError

from qtpy import (
    PYQT5,
    PYQT6,
    PYQT_VERSION,
    PYSIDE2,
    PYSIDE6,
    QtCore,
    QtGui,
    QtWidgets,
)
from qtpy.tests.utils import not_using_conda, using_conda


def test_qtextedit_functions(qtbot, pdf_writer):
    """Test functions mapping for QtWidgets.QTextEdit."""
    assert QtWidgets.QTextEdit.setTabStopWidth
    assert QtWidgets.QTextEdit.tabStopWidth
    assert QtWidgets.QTextEdit.print_
    textedit_widget = QtWidgets.QTextEdit(None)
    textedit_widget.setTabStopWidth(90)
    assert textedit_widget.tabStopWidth() == 90
    print_device, output_path = pdf_writer
    textedit_widget.print_(print_device)
    assert output_path.exists()


def test_qlineedit_functions():
    """Test functions mapping for QtWidgets.QLineEdit"""
    assert QtWidgets.QLineEdit.getTextMargins


def test_what_moved_to_qtgui_in_qt6():
    """Test that we move back what has been moved to QtGui in Qt6"""
    assert QtWidgets.QAction is not None
    assert QtWidgets.QActionGroup is not None
    assert QtWidgets.QFileSystemModel is not None
    assert QtWidgets.QShortcut is not None
    assert QtWidgets.QUndoCommand is not None


def test_qplaintextedit_functions(qtbot, pdf_writer):
    """Test functions mapping for QtWidgets.QPlainTextEdit."""
    assert QtWidgets.QPlainTextEdit.setTabStopWidth
    assert QtWidgets.QPlainTextEdit.tabStopWidth
    assert QtWidgets.QPlainTextEdit.print_
    plaintextedit_widget = QtWidgets.QPlainTextEdit(None)
    plaintextedit_widget.setTabStopWidth(90)
    assert plaintextedit_widget.tabStopWidth() == 90
    print_device, output_path = pdf_writer
    plaintextedit_widget.print_(print_device)
    assert output_path.exists()


def test_QApplication_exec_():
    """Test `QtWidgets.QApplication.exec_`"""
    assert QtWidgets.QApplication.exec_ is not None
    app = QtWidgets.QApplication.instance() or QtWidgets.QApplication(
        [sys.executable, __file__],
    )
    assert app is not None
    QtCore.QTimer.singleShot(100, QtWidgets.QApplication.instance().quit)
    QtWidgets.QApplication.exec_()
    app = QtWidgets.QApplication.instance() or QtWidgets.QApplication(
        [sys.executable, __file__],
    )
    assert app is not None
    QtCore.QTimer.singleShot(100, QtWidgets.QApplication.instance().quit)
    app.exec_()


@pytest.mark.skipif(
    sys.platform == "darwin" and sys.version_info[:2] == (3, 7),
    reason="Stalls on macOS CI with Python 3.7",
)
def test_qdialog_functions(qtbot):
    """Test functions mapping for QtWidgets.QDialog."""
    assert QtWidgets.QDialog.exec_
    dialog = QtWidgets.QDialog(None)
    QtCore.QTimer.singleShot(100, dialog.accept)
    dialog.exec_()


@pytest.mark.skipif(
    sys.platform == "darwin" and sys.version_info[:2] == (3, 7),
    reason="Stalls on macOS CI with Python 3.7",
)
def test_qdialog_subclass(qtbot):
    """Test functions mapping for QtWidgets.QDialog when using a subclass"""
    assert QtWidgets.QDialog.exec_

    class CustomDialog(QtWidgets.QDialog):
        def __init__(self):
            super().__init__(None)
            self.setWindowTitle("Testing")

    assert CustomDialog.exec_
    dialog = CustomDialog()
    QtCore.QTimer.singleShot(100, dialog.accept)
    dialog.exec_()


@pytest.mark.skipif(
    sys.platform == "darwin" and sys.version_info[:2] == (3, 7),
    reason="Stalls on macOS CI with Python 3.7",
)
def test_QMenu_functions(qtbot):
    """Test functions mapping for `QtWidgets.QMenu`."""
    # A window is required for static calls
    window = QtWidgets.QMainWindow()
    menu = QtWidgets.QMenu(window)
    menu.addAction("QtPy")
    menu.addAction("QtPy with a shortcut", QtGui.QKeySequence.UnknownKey)
    menu.addAction(
        QtGui.QIcon(),
        "QtPy with an icon and a shortcut",
        QtGui.QKeySequence.UnknownKey,
    )
    window.show()

    with qtbot.waitExposed(window):
        # Call `exec_` of a `QMenu` instance
        QtCore.QTimer.singleShot(100, menu.close)
        menu.exec_()

        # Call static `QMenu.exec_`
        QtCore.QTimer.singleShot(
            100,
            lambda: qtbot.keyClick(
                QtWidgets.QApplication.widgetAt(1, 1),
                QtCore.Qt.Key_Escape,
            ),
        )
        QtWidgets.QMenu.exec_(menu.actions(), QtCore.QPoint(1, 1))


@pytest.mark.skipif(
    sys.platform == "darwin" and sys.version_info[:2] == (3, 7),
    reason="Stalls on macOS CI with Python 3.7",
)
def test_QToolBar_functions(qtbot):
    """Test `QtWidgets.QToolBar.addAction` compatibility with Qt6 arguments' order."""
    toolbar = QtWidgets.QToolBar()
    toolbar.addAction("QtPy with a shortcut", QtGui.QKeySequence.UnknownKey)
    toolbar.addAction(
        QtGui.QIcon(),
        "QtPy with an icon and a shortcut",
        QtGui.QKeySequence.UnknownKey,
    )


@pytest.mark.skipif(
    PYQT5 and PYQT_VERSION.startswith("5.9"),
    reason="A specific setup with at least sip 4.9.9 is needed for PyQt5 5.9.*"
    "to work with scoped enum access",
)
def test_enum_access():
    """Test scoped and unscoped enum access for qtpy.QtWidgets.*."""
    assert (
        QtWidgets.QFileDialog.AcceptOpen
        == QtWidgets.QFileDialog.AcceptMode.AcceptOpen
    )
    assert (
        QtWidgets.QMessageBox.InvalidRole
        == QtWidgets.QMessageBox.ButtonRole.InvalidRole
    )
    assert QtWidgets.QStyle.State_None == QtWidgets.QStyle.StateFlag.State_None
    assert (
        QtWidgets.QSlider.TicksLeft
        == QtWidgets.QSlider.TickPosition.TicksAbove
    )
    assert (
        QtWidgets.QStyle.SC_SliderGroove
        == QtWidgets.QStyle.SubControl.SC_SliderGroove
    )


def test_opengl_imports():
    """
    Test for presence of QOpenGLWidget.

    QOpenGLWidget was a member of QtWidgets in Qt5, but moved to QtOpenGLWidgets in Qt6.
    QtPy makes QOpenGLWidget available in QtWidgets to maintain compatibility.
    """
    assert QtWidgets.QOpenGLWidget is not None


@pytest.mark.skipif(
    sys.platform == "darwin"
    and sys.version_info[:2] == (3, 7)
    and (PYQT5 or PYSIDE2),
    reason="Crashes on macOS with Python 3.7 with 'Illegal instruction: 4'",
)
@pytest.mark.parametrize("keyword", ["dir", "directory"])
@pytest.mark.parametrize("instance", [True, False])
def test_qfiledialog_dir_compat(tmp_path, qtbot, keyword, instance):
    """
    This function is testing if the decorators that renamed the dir/directory
    keyword are working.

    It may stop working if the Qt bindings do some overwriting of the methods
    in constructor. It should not happen, but the PySide team
    did similar things in the past (like overwriting enum module in
    PySide6==6.3.2).

    keyword: str
        The keyword that should be used in the function call.
    instance: bool
        If True, the function is called on the instance of the QFileDialog,
        otherwise on the class.
    """

    class CloseThread(QtCore.QThread):
        """
        On some implementations the `getExistingDirectory` functions starts own
        event loop that will not trigger QTimer started before the call. Until
        the dialog is closed the main event loop will be stopped.

        Because of this it is required to use the thread to interact with the
        dialog.
        """

        def run(self, allow_restart=True):
            sleep(0.5)
            need_restart = allow_restart
            app = QtWidgets.QApplication.instance()
            for dlg in app.topLevelWidgets():
                if (
                    not isinstance(dlg, QtWidgets.QFileDialog)
                    or dlg.isHidden()
                ):
                    continue
                # "when implemented this I try to use:
                # * dlg.close() - On Qt6 it will close the dialog, but it will
                #   not restart the main event loop.
                # * dlg.accept() - It ends with information thar `accept` and
                #   `reject` of such created dialog can not be called.
                # * accept dialog with enter - It works, but it cannot be
                #   called to early after dialog is shown
                qtbot.keyClick(dlg, QtCore.Qt.Key_Enter)
                need_restart = False
            sleep(0.1)
            for dlg in app.topLevelWidgets():
                # As described above, it may happen that dialog is not closed after first using enter.
                # in such case we call `run` function again. The 0.5s sleep is enough for the second enter to close the dialog.
                if (
                    not isinstance(dlg, QtWidgets.QFileDialog)
                    or dlg.isHidden()
                ):
                    continue
                self.run(allow_restart=False)
                return

            if need_restart:
                self.run()

    # We need to use the `DontUseNativeDialog` option to be able to interact
    # with it from code.
    try:
        opt = QtWidgets.QFileDialog.Option.DontUseNativeDialog
    except AttributeError:
        # old qt5 bindings
        opt = QtWidgets.QFileDialog.DontUseNativeDialog

    kwargs = {
        "caption": "Select a directory",
        keyword: str(tmp_path),
        "options": opt,
    }

    thr = CloseThread()
    thr.start()
    qtbot.waitUntil(thr.isRunning, timeout=1000)
    dlg = QtWidgets.QFileDialog() if instance else QtWidgets.QFileDialog
    dlg.getExistingDirectory(**kwargs)
    qtbot.waitUntil(thr.isFinished, timeout=3000)


def test_qfiledialog_flags_typedef():
    """
    Test existence of `QFlags<Option>` typedef `Options` that was removed from PyQt6
    """
    assert QtWidgets.QFileDialog.Options is not None
    assert QtWidgets.QFileDialog.Options() == QtWidgets.QFileDialog.Option(0)



import pytest


def test_qtremoteobjects():
    """Test the qtpy.QtRemoteObjects namespace"""
    QtRemoteObjects = pytest.importorskip("qtpy.QtRemoteObjects")

    assert QtRemoteObjects.QRemoteObjectAbstractPersistedStore is not None
    assert QtRemoteObjects.QRemoteObjectDynamicReplica is not None
    assert QtRemoteObjects.QRemoteObjectHost is not None
    assert QtRemoteObjects.QRemoteObjectHostBase is not None
    assert QtRemoteObjects.QRemoteObjectNode is not None



import pytest


def test_qtwinextras():
    QtX11Extras = pytest.importorskip("qtpy.QtX11Extras")

    assert QtX11Extras is not None
    # This module doesn't seem to contain any classes
    # See https://doc.qt.io/qt-5/qtx11extras-module.html



#
# Copyright © 2009- The Spyder Development Team
# Copyright © 2014-2015 Colin Duquesnoy
#
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)

"""
**QtPy** is a shim over the various Python Qt bindings. It is used to write
Qt binding independent libraries or applications.

If one of the APIs has already been imported, then it will be used.

Otherwise, the shim will automatically select the first available API (PyQt5, PySide2,
PyQt6 and PySide6); in that case, you can force the use of one
specific bindings (e.g. if your application is using one specific bindings and
you need to use library that use QtPy) by setting up the ``QT_API`` environment
variable.

PyQt5
=====

For PyQt5, you don't have to set anything as it will be used automatically::

    >>> from qtpy import QtGui, QtWidgets, QtCore
    >>> print(QtWidgets.QWidget)

PySide2
======

Set the QT_API environment variable to 'pyside2' before importing other
packages::

    >>> import os
    >>> os.environ['QT_API'] = 'pyside2'
    >>> from qtpy import QtGui, QtWidgets, QtCore
    >>> print(QtWidgets.QWidget)

PyQt6
=====

    >>> import os
    >>> os.environ['QT_API'] = 'pyqt6'
    >>> from qtpy import QtGui, QtWidgets, QtCore
    >>> print(QtWidgets.QWidget)

PySide6
=======

    >>> import os
    >>> os.environ['QT_API'] = 'pyside6'
    >>> from qtpy import QtGui, QtWidgets, QtCore
    >>> print(QtWidgets.QWidget)

"""

import contextlib
import os
import platform
import sys
import warnings

from packaging.version import parse

# Version of QtPy
__version__ = "2.4.1"


class PythonQtError(RuntimeError):
    """Generic error superclass for QtPy."""


class PythonQtWarning(RuntimeWarning):
    """Warning class for QtPy."""


class PythonQtValueError(ValueError):
    """Error raised if an invalid QT_API is specified."""


class QtBindingsNotFoundError(PythonQtError, ImportError):
    """Error raised if no bindings could be selected."""

    _msg = "No Qt bindings could be found"

    def __init__(self):
        super().__init__(self._msg)


class QtModuleNotFoundError(ModuleNotFoundError, PythonQtError):
    """Raised when a Python Qt binding submodule is not installed/supported."""

    _msg = "The {name} module was not found."
    _msg_binding = "{binding}"
    _msg_extra = ""

    def __init__(self, *, name, msg=None, **msg_kwargs):
        global API_NAME
        binding = self._msg_binding.format(binding=API_NAME)
        msg = msg or f"{self._msg} {self._msg_extra}".strip()
        msg = msg.format(name=name, binding=binding, **msg_kwargs)
        super().__init__(msg, name=name)


class QtModuleNotInOSError(QtModuleNotFoundError):
    """Raised when a module is not supported on the current operating system."""

    _msg = "{name} does not exist on this operating system."


class QtModuleNotInQtVersionError(QtModuleNotFoundError):
    """Raised when a module is not implemented in the current Qt version."""

    _msg = "{name} does not exist in {version}."

    def __init__(self, *, name, msg=None, **msg_kwargs):
        global QT5, QT6
        version = "Qt5" if QT5 else "Qt6"
        super().__init__(name=name, version=version)


class QtBindingMissingModuleError(QtModuleNotFoundError):
    """Raised when a module is not supported by a given binding."""

    _msg_extra = "It is not currently implemented in {binding}."


class QtModuleNotInstalledError(QtModuleNotFoundError):
    """Raise when a module is supported by the binding, but not installed."""

    _msg_extra = "It must be installed separately"

    def __init__(self, *, missing_package=None, **superclass_kwargs):
        self.missing_package = missing_package
        if missing_package is not None:
            self._msg_extra += " as {missing_package}."
        super().__init__(missing_package=missing_package, **superclass_kwargs)


# Qt API environment variable name
QT_API = "QT_API"

# Names of the expected PyQt5 api
PYQT5_API = ["pyqt5"]

PYQT6_API = ["pyqt6"]

# Names of the expected PySide2 api
PYSIDE2_API = ["pyside2"]

# Names of the expected PySide6 api
PYSIDE6_API = ["pyside6"]

# Minimum supported versions of Qt and the bindings
QT5_VERSION_MIN = PYQT5_VERSION_MIN = "5.9.0"
PYSIDE2_VERSION_MIN = "5.12.0"
QT6_VERSION_MIN = PYQT6_VERSION_MIN = PYSIDE6_VERSION_MIN = "6.2.0"

QT_VERSION_MIN = QT5_VERSION_MIN
PYQT_VERSION_MIN = PYQT5_VERSION_MIN
PYSIDE_VERSION_MIN = PYSIDE2_VERSION_MIN

# Detecting if a binding was specified by the user
binding_specified = QT_API in os.environ

API_NAMES = {
    "pyqt5": "PyQt5",
    "pyside2": "PySide2",
    "pyqt6": "PyQt6",
    "pyside6": "PySide6",
}
API = os.environ.get(QT_API, "pyqt5").lower()
initial_api = API
if API not in API_NAMES:
    raise PythonQtValueError(
        f"Specified QT_API={QT_API.lower()!r} is not in valid options: "
        f"{API_NAMES}",
    )

is_old_pyqt = is_pyqt46 = False
QT5 = PYQT5 = True
QT4 = QT6 = PYQT4 = PYQT6 = PYSIDE = PYSIDE2 = PYSIDE6 = False

PYQT_VERSION = None
PYSIDE_VERSION = None
QT_VERSION = None

# Unless `FORCE_QT_API` is set, use previously imported Qt Python bindings
if not os.environ.get("FORCE_QT_API"):
    if "PyQt5" in sys.modules:
        API = initial_api if initial_api in PYQT5_API else "pyqt5"
    elif "PySide2" in sys.modules:
        API = initial_api if initial_api in PYSIDE2_API else "pyside2"
    elif "PyQt6" in sys.modules:
        API = initial_api if initial_api in PYQT6_API else "pyqt6"
    elif "PySide6" in sys.modules:
        API = initial_api if initial_api in PYSIDE6_API else "pyside6"

if API in PYQT5_API:
    try:
        from PyQt5.QtCore import (
            PYQT_VERSION_STR as PYQT_VERSION,
        )
        from PyQt5.QtCore import (
            QT_VERSION_STR as QT_VERSION,
        )

        QT5 = PYQT5 = True

        if sys.platform == "darwin":
            macos_version = parse(platform.mac_ver()[0])
            qt_ver = parse(QT_VERSION)
            if macos_version < parse("10.10") and qt_ver >= parse("5.9"):
                raise PythonQtError(
                    "Qt 5.9 or higher only works in "
                    "macOS 10.10 or higher. Your "
                    "program will fail in this "
                    "system.",
                )
            elif macos_version < parse("10.11") and qt_ver >= parse("5.11"):
                raise PythonQtError(
                    "Qt 5.11 or higher only works in "
                    "macOS 10.11 or higher. Your "
                    "program will fail in this "
                    "system.",
                )

            del macos_version
            del qt_ver
    except ImportError:
        API = "pyside2"
    else:
        os.environ[QT_API] = API

if API in PYSIDE2_API:
    try:
        from PySide2 import __version__ as PYSIDE_VERSION  # analysis:ignore
        from PySide2.QtCore import __version__ as QT_VERSION  # analysis:ignore

        PYQT5 = False
        QT5 = PYSIDE2 = True

        if sys.platform == "darwin":
            macos_version = parse(platform.mac_ver()[0])
            qt_ver = parse(QT_VERSION)
            if macos_version < parse("10.11") and qt_ver >= parse("5.11"):
                raise PythonQtError(
                    "Qt 5.11 or higher only works in "
                    "macOS 10.11 or higher. Your "
                    "program will fail in this "
                    "system.",
                )

            del macos_version
            del qt_ver
    except ImportError:
        API = "pyqt6"
    else:
        os.environ[QT_API] = API

if API in PYQT6_API:
    try:
        from PyQt6.QtCore import (
            PYQT_VERSION_STR as PYQT_VERSION,
        )
        from PyQt6.QtCore import (
            QT_VERSION_STR as QT_VERSION,
        )

        QT5 = PYQT5 = False
        QT6 = PYQT6 = True

    except ImportError:
        API = "pyside6"
    else:
        os.environ[QT_API] = API

if API in PYSIDE6_API:
    try:
        from PySide6 import __version__ as PYSIDE_VERSION  # analysis:ignore
        from PySide6.QtCore import __version__ as QT_VERSION  # analysis:ignore

        QT5 = PYQT5 = False
        QT6 = PYSIDE6 = True

    except ImportError:
        raise QtBindingsNotFoundError from None
    else:
        os.environ[QT_API] = API


# If a correct API name is passed to QT_API and it could not be found,
# switches to another and informs through the warning
if initial_api != API and binding_specified:
    warnings.warn(
        f"Selected binding {initial_api!r} could not be found; "
        f"falling back to {API!r}",
        PythonQtWarning,
        stacklevel=2,
    )


# Set display name of the Qt API
API_NAME = API_NAMES[API]

with contextlib.suppress(ImportError, PythonQtError):
    # QtDataVisualization backward compatibility (QtDataVisualization vs. QtDatavisualization)
    # Only available for Qt5 bindings > 5.9 on Windows
    from . import QtDataVisualization as QtDatavisualization  # analysis:ignore


def _warn_old_minor_version(name, old_version, min_version):
    """Warn if using a Qt or binding version no longer supported by QtPy."""
    warning_message = (
        f"{name} version {old_version} is not supported by QtPy. "
        "To ensure your application works correctly with QtPy, "
        f"please upgrade to {name} {min_version} or later."
    )
    warnings.warn(warning_message, PythonQtWarning, stacklevel=2)


# Warn if using an End of Life or unsupported Qt API/binding minor version
if QT_VERSION:
    if QT5 and (parse(QT_VERSION) < parse(QT5_VERSION_MIN)):
        _warn_old_minor_version("Qt5", QT_VERSION, QT5_VERSION_MIN)
    elif QT6 and (parse(QT_VERSION) < parse(QT6_VERSION_MIN)):
        _warn_old_minor_version("Qt6", QT_VERSION, QT6_VERSION_MIN)

if PYQT_VERSION:
    if PYQT5 and (parse(PYQT_VERSION) < parse(PYQT5_VERSION_MIN)):
        _warn_old_minor_version("PyQt5", PYQT_VERSION, PYQT5_VERSION_MIN)
    elif PYQT6 and (parse(PYQT_VERSION) < parse(PYQT6_VERSION_MIN)):
        _warn_old_minor_version("PyQt6", PYQT_VERSION, PYQT6_VERSION_MIN)
elif PYSIDE_VERSION:
    if PYSIDE2 and (parse(PYSIDE_VERSION) < parse(PYSIDE2_VERSION_MIN)):
        _warn_old_minor_version("PySide2", PYSIDE_VERSION, PYSIDE2_VERSION_MIN)
    elif PYSIDE6 and (parse(PYSIDE_VERSION) < parse(PYSIDE6_VERSION_MIN)):
        _warn_old_minor_version("PySide6", PYSIDE_VERSION, PYSIDE6_VERSION_MIN)



import pytest

from qtpy import PYQT5, PYSIDE2


@pytest.mark.skipif(PYSIDE2 or PYQT5, reason="Not available in PySide2/PyQt5")
def test_qtopenglwidgets():
    """Test the qtpy.QtOpenGLWidgets namespace"""
    from qtpy import QtOpenGLWidgets

    assert QtOpenGLWidgets.QOpenGLWidget is not None



import pytest


def test_qtdbus():
    """Test the qtpy.QtDBus namespace"""
    QtDBus = pytest.importorskip("qtpy.QtDBus")

    assert QtDBus.QDBusAbstractAdaptor is not None
    assert QtDBus.QDBusAbstractInterface is not None
    assert QtDBus.QDBusArgument is not None
    assert QtDBus.QDBusConnection is not None



# Copyright © 2009- The Spyder Development Team
# Copyright © 2012- University of North Carolina at Chapel Hill
#                   Luke Campagnola    ('luke.campagnola@%s.com' % 'gmail')
#                   Ogi Moore          ('ognyan.moore@%s.com' % 'gmail')
#                   KIU Shueng Chuan   ('nixchuan@%s.com' % 'gmail')
# Licensed under the terms of the MIT License

"""
Compatibility functions for scoped and unscoped enum access.
"""

from . import PYQT6

if PYQT6:
    import enum

    from . import sip

    def promote_enums(module):
        """
        Search enums in the given module and allow unscoped access.

        Taken from:
        https://github.com/pyqtgraph/pyqtgraph/blob/pyqtgraph-0.12.1/pyqtgraph/Qt.py#L331-L377
        and adapted to also copy enum values aliased under different names.

        """
        class_names = [name for name in dir(module) if name.startswith("Q")]
        for class_name in class_names:
            klass = getattr(module, class_name)
            if not isinstance(klass, sip.wrappertype):
                continue
            attrib_names = [name for name in dir(klass) if name[0].isupper()]
            for attrib_name in attrib_names:
                attrib = getattr(klass, attrib_name)
                if not isinstance(attrib, enum.EnumMeta):
                    continue
                for name, value in attrib.__members__.items():
                    setattr(klass, name, value)



# -----------------------------------------------------------------------------
# Copyright © 2009- The QtPy Contributors
#
# Released under the terms of the MIT License
# (see LICENSE.txt for details)
# -----------------------------------------------------------------------------

"""Provide a CLI to allow configuring developer settings, including mypy."""

# Standard library imports
import argparse
import json
import textwrap


def print_version():
    """Print the current version of the package."""
    import qtpy

    print("QtPy version", qtpy.__version__)


def get_api_status():
    """Get the status of each Qt API usage."""
    import qtpy

    return {name: name == qtpy.API for name in qtpy.API_NAMES}


def generate_mypy_args():
    """Generate a string with always-true/false args to pass to mypy."""
    options = {False: "--always-false", True: "--always-true"}

    apis_active = get_api_status()
    return " ".join(
        f"{options[is_active]}={name.upper()}"
        for name, is_active in apis_active.items()
    )


def generate_pyright_config_json():
    """Generate Pyright config to be used in `pyrightconfig.json`."""
    apis_active = get_api_status()

    return json.dumps(
        {
            "defineConstant": {
                name.upper(): is_active
                for name, is_active in apis_active.items()
            },
        },
    )


def generate_pyright_config_toml():
    """Generate a Pyright config to be used in `pyproject.toml`."""
    apis_active = get_api_status()

    return "[tool.pyright.defineConstant]\n" + "\n".join(
        f"{name.upper()} = {str(is_active).lower()}"
        for name, is_active in apis_active.items()
    )


def print_mypy_args():
    """Print the generated mypy args to stdout."""
    print(generate_mypy_args())


def print_pyright_config_json():
    """Print the generated Pyright JSON config to stdout."""
    print(generate_pyright_config_json())


def print_pyright_config_toml():
    """Print the generated Pyright TOML config to stdout."""
    print(generate_pyright_config_toml())


def print_pyright_configs():
    """Print the generated Pyright configs to stdout."""
    print("pyrightconfig.json:")
    print_pyright_config_json()
    print()
    print("pyproject.toml:")
    print_pyright_config_toml()


def generate_arg_parser():
    """Generate the argument parser for the dev CLI for QtPy."""
    parser = argparse.ArgumentParser(
        description="Features to support development with QtPy.",
    )
    parser.set_defaults(func=parser.print_help)

    parser.add_argument(
        "--version",
        action="store_const",
        dest="func",
        const=print_version,
        help="If passed, will print the version and exit",
    )

    cli_subparsers = parser.add_subparsers(
        title="Subcommands",
        help="Subcommand to run",
        metavar="Subcommand",
    )

    # Parser for the MyPy args subcommand
    mypy_args_parser = cli_subparsers.add_parser(
        name="mypy-args",
        help="Generate command line arguments for using mypy with QtPy.",
        formatter_class=argparse.RawTextHelpFormatter,
        description=textwrap.dedent(
            """
            Generate command line arguments for using mypy with QtPy.

            This will generate strings similar to the following
            which help guide mypy through which library QtPy would have used
            so that mypy can get the proper underlying type hints.

                --always-false=PYQT5 --always-false=PYQT6 --always-true=PYSIDE2 --always-false=PYSIDE6

            It can be used as follows on Bash or a similar shell:

                mypy --package mypackage $(qtpy mypy-args)
            """,
        ),
    )
    mypy_args_parser.set_defaults(func=print_mypy_args)

    # Parser for the Pyright config subcommand
    pyright_config_parser = cli_subparsers.add_parser(
        name="pyright-config",
        help="Generate Pyright config for using Pyright with QtPy.",
        formatter_class=argparse.RawTextHelpFormatter,
        description=textwrap.dedent(
            """
            Generate Pyright config for using Pyright with QtPy.

            This will generate config sections to be included in a Pyright
            config file (either `pyrightconfig.json` or `pyproject.toml`)
            which help guide Pyright through which library QtPy would have used
            so that Pyright can get the proper underlying type hints.

            """,
        ),
    )
    pyright_config_parser.set_defaults(func=print_pyright_configs)

    return parser


def main(args=None):
    """Run the development CLI for QtPy."""
    parser = generate_arg_parser()
    parsed_args = parser.parse_args(args=args)

    reserved_params = {"func"}
    cleaned_args = {
        key: value
        for key, value in vars(parsed_args).items()
        if key not in reserved_params
    }
    parsed_args.func(**cleaned_args)



import pytest


def test_qtstatemachine():
    """Test the qtpy.QtStateMachine namespace"""
    QtStateMachine = pytest.importorskip("qtpy.QtStateMachine")

    assert QtStateMachine.QAbstractState is not None
    assert QtStateMachine.QAbstractTransition is not None
    assert QtStateMachine.QEventTransition is not None
    assert QtStateMachine.QFinalState is not None
    assert QtStateMachine.QHistoryState is not None
    assert QtStateMachine.QKeyEventTransition is not None
    assert QtStateMachine.QMouseEventTransition is not None
    assert QtStateMachine.QSignalTransition is not None
    assert QtStateMachine.QState is not None



import pytest

from qtpy import QT6
from qtpy.tests.utils import using_conda


@pytest.mark.skipif(
    QT6 and using_conda(),
    reason="QPositioning bindings not included in Conda qt-main >= 6.4.3.",
)
def test_qtpositioning():
    """Test the qtpy.QtPositioning namespace"""
    from qtpy import QtPositioning

    assert QtPositioning.QGeoAddress is not None
    assert QtPositioning.QGeoAreaMonitorInfo is not None
    assert QtPositioning.QGeoAreaMonitorSource is not None
    assert QtPositioning.QGeoCircle is not None
    assert QtPositioning.QGeoCoordinate is not None
    assert QtPositioning.QGeoLocation is not None
    assert QtPositioning.QGeoPath is not None
    # CI for 3.7 uses Qt 5.9
    # assert QtPositioning.QGeoPolygon is not None  # New in Qt 5.10
    assert QtPositioning.QGeoPositionInfo is not None
    assert QtPositioning.QGeoPositionInfoSource is not None
    # QGeoPositionInfoSourceFactory is not available in PyQt
    # assert QtPositioning.QGeoPositionInfoSourceFactory is not None  # New in Qt 5.2
    # assert QtPositioning.QGeoPositionInfoSourceFactoryV2 is not None  # New in Qt 5.14
    assert QtPositioning.QGeoRectangle is not None
    assert QtPositioning.QGeoSatelliteInfo is not None
    assert QtPositioning.QGeoSatelliteInfoSource is not None
    assert QtPositioning.QGeoShape is not None
    assert QtPositioning.QNmeaPositionInfoSource is not None



import pytest
from packaging.version import parse

from qtpy import PYSIDE2, PYSIDE_VERSION


def test_qtconcurrent():
    """Test the qtpy.QtConcurrent namespace"""
    QtConcurrent = pytest.importorskip("qtpy.QtConcurrent")

    assert QtConcurrent.QtConcurrent is not None

    if PYSIDE2 and parse(PYSIDE_VERSION) >= parse("5.15.2"):
        assert QtConcurrent.QFutureQString is not None
        assert QtConcurrent.QFutureVoid is not None
        assert QtConcurrent.QFutureWatcherQString is not None
        assert QtConcurrent.QFutureWatcherVoid is not None



"""Test Qsci."""

import pytest

from qtpy import PYSIDE2, PYSIDE6
from qtpy.tests.utils import using_conda


@pytest.mark.skipif(
    PYSIDE2 or PYSIDE6 or using_conda(),
    reason="Qsci bindings not available under PySide 2/6 and conda installations",
)
def test_qsci():
    """Test the qtpy.Qsci namespace"""
    Qsci = pytest.importorskip("qtpy.Qsci")
    assert Qsci.QSCINTILLA_VERSION is not None
    assert Qsci.QSCINTILLA_VERSION_STR is not None
    assert Qsci.QsciAPIs is not None
    assert Qsci.QsciAbstractAPIs is not None
    assert Qsci.QsciCommand is not None
    assert Qsci.QsciCommandSet is not None
    assert Qsci.QsciDocument is not None
    assert Qsci.QsciLexer is not None
    assert Qsci.QsciLexerAVS is not None
    assert Qsci.QsciLexerBash is not None
    assert Qsci.QsciLexerBatch is not None
    assert Qsci.QsciLexerCMake is not None
    assert Qsci.QsciLexerCPP is not None
    assert Qsci.QsciLexerCSS is not None
    assert Qsci.QsciLexerCSharp is not None
    assert Qsci.QsciLexerCoffeeScript is not None
    assert Qsci.QsciLexerCustom is not None
    assert Qsci.QsciLexerD is not None
    assert Qsci.QsciLexerDiff is not None
    assert Qsci.QsciLexerFortran is not None
    assert Qsci.QsciLexerFortran77 is not None
    assert Qsci.QsciLexerHTML is not None
    assert Qsci.QsciLexerIDL is not None
    assert Qsci.QsciLexerJSON is not None
    assert Qsci.QsciLexerJava is not None
    assert Qsci.QsciLexerJavaScript is not None
    assert Qsci.QsciLexerLua is not None
    assert Qsci.QsciLexerMakefile is not None
    assert Qsci.QsciLexerMarkdown is not None
    assert Qsci.QsciLexerMatlab is not None
    assert Qsci.QsciLexerOctave is not None
    assert Qsci.QsciLexerPO is not None
    assert Qsci.QsciLexerPOV is not None
    assert Qsci.QsciLexerPascal is not None
    assert Qsci.QsciLexerPerl is not None
    assert Qsci.QsciLexerPostScript is not None
    assert Qsci.QsciLexerProperties is not None
    assert Qsci.QsciLexerPython is not None
    assert Qsci.QsciLexerRuby is not None
    assert Qsci.QsciLexerSQL is not None
    assert Qsci.QsciLexerSpice is not None
    assert Qsci.QsciLexerTCL is not None
    assert Qsci.QsciLexerTeX is not None
    assert Qsci.QsciLexerVHDL is not None
    assert Qsci.QsciLexerVerilog is not None
    assert Qsci.QsciLexerXML is not None
    assert Qsci.QsciLexerYAML is not None
    assert Qsci.QsciMacro is not None
    assert Qsci.QsciPrinter is not None
    assert Qsci.QsciScintilla is not None
    assert Qsci.QsciScintillaBase is not None
    assert Qsci.QsciStyle is not None
    assert Qsci.QsciStyledText is not None
